////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2010-2011 Kentaro Sekimoto  All rights reserved.
////////////////////////////////////////////////////////////////////////////
#include <tinyhal.h>
#include "R61580.h"

void P16_Initialize(void);
void LCD_nRESET(int n);
void P16_LCD_cmd(volatile unsigned short index, volatile unsigned short data);
void P16_LCD_index(volatile unsigned short index);
void P16_LCD_dat(volatile unsigned short data);
unsigned short P16_LCD_read_dat(volatile unsigned short index);

R61580_Driver g_R61580_Driver;

static void delayms(volatile UINT32 n)
{
	n *= 50000;
	while (n-- > 0) ;
}

#if 0
static
void display_area_set(int xs, int ys, int xe, int ye)
{
    P16_LCD_cmd(0x0050, xs);	// Horizontal GRAM Start Address
    P16_LCD_cmd(0x0051, xe);	// Horizontal GRAM End Address
    P16_LCD_cmd(0x0052, ys);	// Vertical GRAM Start Address
    P16_LCD_cmd(0x0053, ye);	// Vertical GRAM End Address
    P16_LCD_cmd(0x0020, xs);	// GRAM horizontal Address
    P16_LCD_cmd(0x0021, ys);	// GRAM Vertical Address
	return;
}
#endif

BOOL R61580_Driver::Initialize()
{
	unsigned short id;
	P16_Initialize();

	LCD_nRESET(1);
	delayms(10);		// Delay 1ms
	LCD_nRESET(0);
	delayms(100);	// Delay 10ms // This delay time is necessary
	LCD_nRESET(1);
	delayms(200);	// Delay 50ms

    P16_LCD_cmd(0x0000, 0x0000);
    P16_LCD_cmd(0x0000, 0x0000);
    delayms(100); // Delay 100ms
    P16_LCD_cmd(0x0000, 0x0000);
    P16_LCD_cmd(0x0000, 0x0000);
    P16_LCD_cmd(0x0000, 0x0000);
    P16_LCD_cmd(0x0000, 0x0000);
    // Setup display
    P16_LCD_cmd(0x00A4, 0x0001); // CALB=1
    delayms(200); // Delay 100ms
	id = P16_LCD_read_dat(0x0000);
    P16_LCD_cmd(0x0060, 0xA700); // Driver Output Control
    P16_LCD_cmd(0x0008, 0x0808); // Display Control BP=8, FP=8
	// Gamma Setting
    P16_LCD_cmd(0x0030, 0x0203); // y control
    P16_LCD_cmd(0x0031, 0x080f); // y control
    P16_LCD_cmd(0x0032, 0x0401); // y control
    P16_LCD_cmd(0x0033, 0x050b); // y control
    P16_LCD_cmd(0x0034, 0x3330); // y control
    P16_LCD_cmd(0x0035, 0x0b05); // y control
    P16_LCD_cmd(0x0036, 0x0005); // y control
    P16_LCD_cmd(0x0037, 0x0f08); // y control
    P16_LCD_cmd(0x0038, 0x0302); // y control
    P16_LCD_cmd(0x0039, 0x3033); // y control
	// Power Setting
    P16_LCD_cmd(0x0090, 0x0018); // 80Hz
    P16_LCD_cmd(0x0010, 0x0530); // Power Control
    P16_LCD_cmd(0x0011, 0x0237);
    P16_LCD_cmd(0x0012, 0x01BF);
    P16_LCD_cmd(0x0013, 0x1000);
    delayms(200); // Delay 200ms
    P16_LCD_cmd(0x0001, 0x0100);
    P16_LCD_cmd(0x0002, 0x0200);
    P16_LCD_cmd(0x0003, 0x1030);
    P16_LCD_cmd(0x0009, 0x0001);
    P16_LCD_cmd(0x000A, 0x0008);
    P16_LCD_cmd(0x000C, 0x0000);
    P16_LCD_cmd(0x000D, 0xD000);
    P16_LCD_cmd(0x000E, 0x0030);
    P16_LCD_cmd(0x000F, 0x0000);
    P16_LCD_cmd(0x0020, 0x0000);
    P16_LCD_cmd(0x0021, 0x0000);
    P16_LCD_cmd(0x0029, 0x002e);
    P16_LCD_cmd(0x0050, 0x0000);
    P16_LCD_cmd(0x0051, 0x00EF);
    P16_LCD_cmd(0x0052, 0x0000);
    P16_LCD_cmd(0x0053, 0x013F);
    P16_LCD_cmd(0x0061, 0x0001);
    P16_LCD_cmd(0x006A, 0x0000);
    P16_LCD_cmd(0x0080, 0x0000);
    P16_LCD_cmd(0x0081, 0x0000);
    P16_LCD_cmd(0x0082, 0x005F);
    P16_LCD_cmd(0x0093, 0x0701);
    P16_LCD_cmd(0x0007, 0x0100);
    delayms(400); // Delay 1ms

#if 1
    P16_LCD_index(0x0022);
    unsigned int i,n,m;
    unsigned short color[8]={0xf800,0x07e0,0x001f,0x07ff,0xf81f,0xffe0,0x0000,0xffff};
    for(n=0;n<8;n++)
    	for(i=0;i<40;i++)
    		for(m=0;m<240;m++)
    			P16_LCD_dat(0xffff);

    for(n=0;n<8;n++)
    	for(i=0;i<40;i++)
    		for(m=0;m<240;m++)
    			P16_LCD_dat(color[n]);
#endif
}

BOOL R61580_Driver::Uninitialize()
{
}

void R61580_Driver::PowerSave(BOOL On)
{
}

void R61580_Driver::Clear()
{
	UINT32 x;
	//display_area_set(0, 0, R61580_PWX-1, R61580_PWY-1);
	g_R61580_Driver.m_cx = 0;
	g_R61580_Driver.m_cy = 0;
	P16_LCD_cmd(0x0020, 0);
	P16_LCD_cmd(0x0021, 0);
	P16_LCD_index(0x0022);
	for (x = 0; x < (g_R61580_Config.Width * g_R61580_Config.Height); x++){
		P16_LCD_dat(0x0000);
	}
}

void R61580_Driver::BitBltEx( int x, int y, int width, int height, UINT32 data[] )
{
	int i, j;
	UINT16 v1, v2;
	UINT16 *pdata = (UINT16 *)data;
    pdata += (y * g_R61580_Config.Width + x);
    for (j = 0; j < height; j ++) {
        P16_LCD_cmd(0x0020, x + i);
        P16_LCD_cmd(0x0021, y + j);
        P16_LCD_index(0x0022);
        for (i = 0; i < width; i ++) {
            P16_LCD_dat(*(pdata + i));
        }
        pdata += g_R61580_Config.Width.Width;
    }
}

void R61580_Driver::BitBltEx565(int x, int y, int width, int height, UINT32 data[])
{
	int i, j;
	UINT16 v1, v2;
	UINT16 *pdata = (UINT16 *)data;
    //pdata += (y * g_R61580_Config.Width + x);
    for (j = 0; j < height; j ++) {
        P16_LCD_cmd(0x0020, x + i);
        P16_LCD_cmd(0x0021, y + j);
        P16_LCD_index(0x0022);
        for (i = 0; i < width; i ++) {
            P16_LCD_dat(*(pdata + i));
        }
        pdata += g_R61580_Config.Width.Width;
    }
}

void R61580_Driver::WriteChar_Color(unsigned char c, UINT8 cx, UINT8 cy, UINT16 fgcol, UINT16 bgcol)
{
	UINT8 x, y;
	UINT16 col;
	UINT16 sx = cx * Font_Width() + g_R61580_Config.TextSx;
	UINT16 sy = cy * Font_Height() + g_R61580_Config.TextSy;

	unsigned char *font = (unsigned char *)Font_GetGlyph(c);
	for (y = 0; y < Font_Height(); y++) {
		P16_LCD_cmd(0x0020, sx);
		P16_LCD_cmd(0x0021, sy + y);
		P16_LCD_index(0x0022);
		for (x = 0; x < Font_Width(); x++) {
			if (font[y] & (0x80 >> x)) {
				col = fgcol;
			} else {
				col = bgcol;
			}
			P16_LCD_dat(col);
		}
	}
}

void R61580_Driver::WriteChar(unsigned char c, int cx, int cy)
{
	WriteChar_Color(c, (UINT8)cx, (UINT8)cy, g_R61580_Config.fcol, g_R61580_Config.bcol);
}

void R61580_Driver::WriteFormattedChar(unsigned char ch)
{
	UINT16 colums = g_R61580_Config.Width/Font_Width();
	UINT16 rows = g_R61580_Config.Height/Font_Height();
	if (ch == 0xc) {
		Clear();
		g_R61580_Driver.m_cx = 0;
		g_R61580_Driver.m_cy = 0;
	} else if (ch == '\n') {
		g_R61580_Driver.m_cy++;
		if (g_R61580_Driver.m_cy == rows) {
			g_R61580_Driver.m_cy = 0;
		}
	} else if (ch == '\r') {
		g_R61580_Driver.m_cx = 0;
	} else {
		WriteChar(ch, g_R61580_Driver.m_cx, g_R61580_Driver.m_cy);
		g_R61580_Driver.m_cx ++;
		if (g_R61580_Driver.m_cx == colums) {
			g_R61580_Driver.m_cx = 0;
			g_R61580_Driver.m_cy ++;
			if (g_R61580_Driver.m_cy == rows) {
				g_R61580_Driver.m_cy = 0;
			}
		}
	}
}
