/**
* @file          hw_support.c
* @brief         hardware support function
* @author        jialili
* @version       1.00
* @date          2010/08/20
*                  Copyright 2010 FUJITSU ELECTRONICS INC.  LIMITED
**/
////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2012 Kentaro Sekimoto  All rights reserved.
////////////////////////////////////////////////////////////////////////////
/*
 * FM3_usb_hw.cpp
 *
 *  Created on: 2012/01/08
 *      Author: Kentaro Sekimoto
 */
#include <tinyhal.h>
#include "..\FM3.h"

#ifndef BIT_SET
#define BIT_SET 1
#endif
#ifndef BIT_CLR
#define BIT_CLR 0
#endif
#ifndef REG_R(reg)
#define REG_R(reg)                (reg)             /* read from register */
#endif
#ifndef REG_W(reg, value)
#define REG_W(reg, value)         ((reg) = (value)) /* write to register  */
#endif

#define UPCR2_UPOWT_INIT_VALUE     (0x07)       /* initial value of UPCR2 register's UPOWT bits    */
#define UPCR3_UPLLK_INIT_VALUE     (0x00)       /* initial value of UPCR3 register's UPLLK bits    */
#ifdef MB9BF618T
#ifdef CLK_16M
#define UPCR4_UPLLN_INIT_VALUE     (0x0E)       /* initial value of UPCR4 register's UPLLN bits    */
#define UPCR5_UPLLN_INIT_VALUE     (0x04)       /* initial value of UPCR4 register's UPLLM bits    */
#else
#define UPCR4_UPLLN_INIT_VALUE     (0x3B)       /* initial value of UPCR4 register's UPLLN bits    */
#define UPCR5_UPLLN_INIT_VALUE     (0x04)       /* initial value of UPCR4 register's UPLLM bits    */
#endif
#else
#ifdef CLK_16M
#define UPCR4_UPLLN_INIT_VALUE     (0x05)       /* initial value of UPCR4 register's UPLLN bits    */
#else
#define UPCR4_UPLLN_INIT_VALUE     (0x17)       /* initial value of UPCR4 register's UPLLN bits    */
#endif
#endif
#define CONDITION_PID           (0)         /* select alternating data toggle mode */
#define NO_CONDITION_PID        (1)         /* select data toggle mode */
#define BUS_POWER               (1)         /* bus-power */
#define SELF_POWER              (0)         /* self-power */
#define HARD_AUTO_CLEAR         (1)         /* STALL is cleared automatically by hardware */

#ifdef MB9BF618T
void FM3_USB_Clock_Initialize(void)
{
    /* disable USB clock */
    REG_W(bFM3_USBETHERNETCLK_UCCR_UCEN0, BIT_CLR);
    /* wait for USB clock stop */
    while (REG_R(bFM3_USBETHERNETCLK_UCCR_UCEN0) != BIT_CLR) ;
    /* disable USB-PLL clock */
    REG_W(bFM3_USBETHERNETCLK_UPCR1_UPLLEN, BIT_CLR);
    /* select PLL macro clock */
    REG_W(bFM3_USBETHERNETCLK_UCCR_UCSEL0, BIT_SET);
    /* select main clock as input clock */
    REG_W(bFM3_USBETHERNETCLK_UPCR1_UPINC, BIT_CLR);
    /* select clock stabilization time */
    REG_W(FM3_USBETHERNETCLK->UPCR2, UPCR2_UPOWT_INIT_VALUE);
    /* USB-PLL=Fin*N/K -> 96MHz=4MHz*24/1 */
    /* USB-PLL clock configuration register(K) initialize */
    REG_W(FM3_USBETHERNETCLK->UPCR3, UPCR3_UPLLK_INIT_VALUE);        // 4MHz:0, 16MHz:0
    /* USB-PLL clock configuration register(N) initialize */
    REG_W(FM3_USBETHERNETCLK->UPCR4, UPCR4_UPLLN_INIT_VALUE);        // 4MHz:60, 16Mz:24
    /* USB-PLL clock configuration register(N) initialize */
    REG_W(FM3_USBETHERNETCLK->UPCR5, UPCR5_UPLLN_INIT_VALUE);        // 4MHz:5,  16Mz:6
    /* USB-PLL clock stabilize interrupt disable  */
    REG_W(bFM3_USBETHERNETCLK_UPINT_ENR_UPCSE, BIT_CLR);
    /* enable USB-PLL clock */
    REG_W(bFM3_USBETHERNETCLK_UPCR1_UPLLEN, BIT_SET);
    /* wait for USB-PLL clock ready */
    while (REG_R(bFM3_USBETHERNETCLK_UP_STR_UPRDY) == BIT_CLR) ;
    /* enable USB clock */
    REG_W(bFM3_USBETHERNETCLK_UCCR_UCEN0, BIT_SET);
    /* wait for 5 cycle */
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    /* enable USB controller */
    REG_W(bFM3_USBETHERNETCLK_USBEN0_USBEN0, BIT_SET);
    return;
}
#endif

#ifdef MB9BF506N
void FM3_USB_Clock_Initialize(void)
{
    /* disable USB clock */
    REG_W(bFM3_USBCLK_UCCR_UCEN, BIT_CLR);
    /* wait for USB clock stop */
    while (REG_R(bFM3_USBCLK_UCCR_UCEN) != BIT_CLR) ;
    /* disable USB-PLL clock */
    REG_W(bFM3_USBCLK_UPCR1_UPLLEN, BIT_CLR);
    /* select PLL macro clock */
    REG_W(bFM3_USBCLK_UCCR_UCSEL, BIT_SET);
    /* select main clock as input clock */
    REG_W(bFM3_USBCLK_UPCR1_UPINC, BIT_CLR);
    /* select clock stabilization time */
    REG_W(FM3_USBCLK->UPCR2, UPCR2_UPOWT_INIT_VALUE);
    /* USB-PLL=Fin*N/K -> 96MHz=4MHz*24/1 */
    /* USB-PLL clock configuration register(K) initialize */
    REG_W(FM3_USBCLK->UPCR3, UPCR3_UPLLK_INIT_VALUE);        // 4MHz:1, 16MHz:1
    /* USB-PLL clock configuration register(N) initialize */
    REG_W(FM3_USBCLK->UPCR4, UPCR4_UPLLN_INIT_VALUE);        // 4MHz:24, 16Mz:6
    /* USB-PLL clock stabilize interrupt disable  */
    REG_W(bFM3_USBCLK_UPINT_ENR_UPCSE, BIT_CLR);
    /* enable USB-PLL clock */
    REG_W(bFM3_USBCLK_UPCR1_UPLLEN, BIT_SET);
    /* wait for USB-PLL clock ready */
    while (REG_R(bFM3_USBCLK_UP_STR_UPRDY) == BIT_CLR) ;
    /* enable USB clock */
    REG_W(bFM3_USBCLK_UCCR_UCEN, BIT_SET);
    /* wait for 5 cycle */
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    /* enable USB controller */
    REG_W(bFM3_USBCLK_USBEN_USBEN, BIT_SET);
    return;
}
#endif

void FM3_USB_Port_Initialize(void)
{
    REG_W(bFM3_GPIO_PFR6_P0, 0);    // P60 GPIO
    REG_W(bFM3_GPIO_DDR6_P0, 0);    // P60 input for checking VBUS
    REG_W(bFM3_GPIO_DDR6_P1, 1);    // P61 outout for controlling pullup.
}
